
package regtopToBusTUC.Holder;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.TreeMap;

import regtopToBusTUC.ConvertAllRegTop;
import regtopToBusTUC.Util;

/**
	Class for the prolog predicates passes3 and ntourstops.<BR>
	Prune away duplicate tdax information. <br>
	Different tdax numbers with equal constants are converted to only one tdax. 

	@author Tore Bruland
	@author Rune Sætre
	@since 111219
 */

public class RegpasHolder {
	private static final String OUTPUT_ENCODING = ConvertAllRegTop.OUTPUT_ENCODING;
	/**
	 * @param bustucOutputRootFolder Where to write the regxxx.pl files (for bustuc/db/tables)
	 */
	public RegpasHolder(String bustucOutputRootFolder) {
		ntourstops = new ArrayList<String>();
		passes3 = new ArrayList<String>();
		tdaTab = new ArrayList<TdaRec>();	
		directory = bustucOutputRootFolder;
		noDup = new ArrayList<TdaSegment> ();
		tdaxIndex = new TreeMap<String,String>();
	}//CONSTRUCTOR

	/**
		instance is used to hold several filesets<br>
		method is called before each new fileset is read 
	 */		
	public void clear() {
		ntourstops.clear();
		passes3.clear();
		directory="";
		tdaTab.clear();
		noDup.clear();
		tdaxIndex.clear();
	}//clear


	public void debug(int level, String message){
		ConvertAllRegTop.debug(level, message);
	} // debug
	/**
		collects data from the tix file.
	 */		
	public void setTix(String adm, String iTdax, String iNstat) {

		String oTdax=Util.trimZero(iTdax);
		String nstat=Util.trimZero(iNstat);	
		String admcode = Util.trimZero(adm);
		String tdax = admcode.trim()+oTdax.trim();
		//System.out.println("tdax "+tdax);
		if (tdax.equals("3811") || tdax.equals("6765")) {
			System.out.println("skal være like. tdax: "+tdax+"  nstat:"+nstat);
		}
		// finn segment
		//System.out.println("debug<"+nstat+">");
		if(!oTdax.equals("null")){
			
		int pos = Integer.parseInt(oTdax.trim());
		//		System.out.println(pos);
		int antall = Integer.parseInt(nstat.trim());
		TdaSegment temp = new TdaSegment(tdax); // new TdaSegment(tdax);  %% TA-110216
		for (int j=0;j<antall;j++) {
			if ( j+pos < 1 ){	// RS-120120
				debug(0, "ERROR: Antall="+antall+", pos="+pos+", size="+tdaTab.size()+", and TDA Segment["+antall+pos+"]:"+tdaTab.get(antall+pos) );
			}else{
				// add t
				try{
					temp.theTab.add(tdaTab.get(j+pos-1));	
				}catch(Exception e){
					System.err.println(j+pos-1);
					System.err.println("tdax "+tdax);
					System.exit(0);
				}
				
			}
		}
		if (tdax.equals("3811") || tdax.equals("6765")) {
			debug(0, ""+temp );
		}
		int i=0;
		int dupNr=-1;
		//System.out.println("the size:"+noDup.size());
		while (i<noDup.size() && dupNr == -1 ) {
			if ( noDup.get(i).equalTo(temp) ) {
				//				System.out.println("adding duplicateNr");
				//System.out.println("duplicate found. "+temp.tdax+" and "+noDup.get(i).tdax + ", the size:"+noDup.size());
				dupNr = i;
				tdaxIndex.put(tdax,noDup.get(i).tdax); // lag konverteringstabell for tdax nummer som fjernes
			}
			i++;
		}

		// duplikat ?
		TdaRec tempRec;
		String passKey;
		if ( dupNr==-1) {
			for (int j=0; j<temp.theTab.size();j++) {
				tempRec = temp.theTab.get(j);
				passKey = "passes4("+temp.tdax+ "," + tempRec.statnr+"," + theHpl.getStatid(tempRec.statnr)+ ","+(j+1)+","+tempRec.arr+","+tempRec.dep+").";
				// TA-100226
				if ( !passes3.contains(passKey) ) { 
					passes3.add(passKey);
				}else{
					//					System.out.println("passes3");
				}
				//if new stop-key
			}//for each stop
			String key = "ntourstops("+tdax+","+nstat+").";
			if ( !ntourstops.contains(key) ) {
				ntourstops.add(key);
			}else{
				//				System.out.println("ntourstops");
			}
			noDup.add(temp);
		} else {
			String key = "ntourstops("+noDup.get(dupNr).tdax+","+nstat+")."; // get the tdax from orginal
			if ( !ntourstops.contains(key) ) {
				ntourstops.add(key);
			}else{
				//				System.out.println("ntourstop2");
			}
			//if new key for ntourstops
		}
		}
		//if dup==-1, else dup found?
	}
	//setTix
	/**
		collects data from the tda file.
	 */		
	public void setTda(String iStatnr, String iArr, String iDep) {
		String arr = Util.trimZero(iArr);
		String dep = Util.trimZero(iDep);
		tdaTab.add(new TdaRec(iStatnr,arr,dep));
	}

	/**
		Pointer to a ReghplHolder class. getStatid method is used in setTix method 
	 */
	public void setReghplHolder(ReghplHolder iHpl) {
		theHpl=iHpl;
	}

	/**
	 *	write the records of the passes3 and ntourstops predicates to the proper directory and filename regpas.pl
	 */	
	public void writePredicates() {
		//System.out.println("ntours antall="+ntourstops.size());
		System.out.println( "TimeStamp: "+ new Timestamp(new Date().getTime()) );
		try {
			BufferedWriter outFile = new BufferedWriter(
					new OutputStreamWriter(new FileOutputStream(directory+File.separator+"regpas.pl"), OUTPUT_ENCODING));

			if(ConvertAllRegTop.OUTPUT_ENCODING.equals("utf-8"))
				outFile.write("/* -*- Mode:Prolog; coding:utf-8; -*- */\n");
			else
				outFile.write("/* -*- Mode:Prolog; coding:iso-8859-1; -*- */\n");

			outFile.write("%% Generated by regtopToBusTUC.RegpasHolder.java on "+ new Timestamp(new Date().getTime()) +"\n\n");
			outFile.write("%% PASSES4 (TOURTYPE, STATION_NR, STATION, SEQ_NR, ARR_DELAY, DEP_DELAY)\n");
			outFile.write("%%    Example: passes4(17326,16011035,anton_grevskotts_vei,22,15,16).\n\n");
			outFile.write("%% NTOURSTOPS (TOURTYPE,LENGTH). Tourtype almost matches the linenumber in regpas.pl and FILE.TDA or FILE.TMS\n");
			outFile.write("%%    Defines the length (number of stations) inthe tourtype, e.g. ntourstops(12713,5).\n");

			outFile.write("\n\n");
			outFile.write("%% PASSES4 (TOURTYPE, STATION NUMBER, STATION, SEQNUMBER, ARR DELAY, DEP DELAY)\n");
			for (Iterator<String> iter=passes3.iterator();iter.hasNext(); ) {
				outFile.write(iter.next()+"\n");
			}				
			for (Iterator<String> iter=ntourstops.iterator();iter.hasNext(); ) {
				outFile.write(iter.next()+"\n");
			}
			outFile.close();
		} catch (IOException e ) {
			System.out.println("********** IOException "+e);
			e.printStackTrace();
		}		
	}//writePredicates

	/**controls the duplicate tdax numbers
	 */		
	public ArrayList<TdaSegment> noDup;

	/**holds record for prolog predicate ntourstops
	 */		
	public ArrayList<String> ntourstops;

	/**holds record for prolog predicate passes3
	 */		
	public ArrayList<String> passes3;

	/**		holds tdaRec redords, tdax number point to start in table and nstat indicate how many records are used.
	 */	
	public ArrayList<TdaRec> tdaTab;

	
	/**	name of the directory
	 */		
	public String directory;

	// pointer
	private ReghplHolder theHpl;

	/**
		Converting table for old tdax numbers
	 */	
	public TreeMap<String,String> tdaxIndex;
} // RegpasHolder

class TdaRec {
	public TdaRec(String iStatnr, String iArr, String iDep) {
		statnr=iStatnr;
		arr=iArr;
		dep=iDep;
	}
	public String toString() {
		return statnr+","+arr+","+dep;
	}
	public boolean equalTo(TdaRec iObj) {
		return (statnr.equals(iObj.statnr) && arr.equals(iObj.arr) && dep.equals(iObj.dep) );
	}
	String statnr, arr, dep;
}//Class TdaRec

class TdaSegment {
	public TdaSegment(String iTdax) {
		theTab = new ArrayList<TdaRec>();
		tdax = iTdax;
	}//TdaSegment CONSTRUCTOR

	public boolean equalTo(TdaSegment iObj) {
		if ( theTab.size() != iObj.theTab.size() ) return false;
		boolean isEqual=true;
		int i=0;
		while (isEqual && i<theTab.size()) {
			isEqual = theTab.get(i).equalTo(iObj.theTab.get(i));
			i++;
		}
		return isEqual;
	}//equalTo

	public String toString() {
		String temp = "tdaSegment(\ntdax:"+tdax+"\n";
		for (Iterator<TdaRec> iter=theTab.iterator(); iter.hasNext(); ) {
			temp += iter.next()+"\n";
		}
		temp += ").";
		return temp;
	}

	String tdax;
	ArrayList<TdaRec> theTab;
}//class TdaSegment
